package com.ruoyi.web.controller.school;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.Result;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.school.domain.SysClass;
import com.ruoyi.school.domain.req.SysClassEditReq;
import com.ruoyi.school.domain.req.SysClassQueryReq;
import com.ruoyi.school.domain.req.SysClassSaveReq;
import com.ruoyi.school.service.SysClassService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 班级管理Controller
 *
 * @author ruoyi
 * @date 2024-04-22
 */
@Api(tags = "班级管理")
@RestController
@RequestMapping("/school/class")
public class SysClassController extends BaseController {
    @Autowired
    private SysClassService sysClassService;

    /**
     * 查询班级管理列表
     */
    @ApiOperation("获取班级管理列表")
    @PreAuthorize("@ss.hasPermi('school:class:list')")
    @GetMapping("/list")
    public Result<TableDataInfo<SysClass>> list(SysClassQueryReq sysClassQueryReq) {
        startPage();
        List<SysClass> list = sysClassService.list(SysClass.queryBuild(sysClassQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出班级管理列表
     */
    @ApiOperation("导出班级管理列表")
    @PreAuthorize("@ss.hasPermi('school:class:export')")
    @Log(title = "班级管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysClass sysClass) {
        List<SysClass> list = sysClassService.list(sysClass);
        ExcelUtil<SysClass> util = new ExcelUtil<SysClass>(SysClass.class);
        util.exportExcel(response, list, "班级管理数据");
    }

    /**
     * 获取班级管理详细信息
     */
    @ApiOperation("获取班级管理详细信息")
    @PreAuthorize("@ss.hasPermi('school:class:query')")
    @GetMapping(value = "/{classId}")
    @ApiImplicitParam(name = "classId", value = "classId", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<SysClass> getInfo(@PathVariable("classId") Long classId) {
        return Result.success(sysClassService.getById(classId));
    }

    /**
     * 新增班级管理
     */
    @PreAuthorize("@ss.hasPermi('school:class:add')")
    @Log(title = "班级管理", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增班级管理")
    public Result<String> add(@RequestBody SysClassSaveReq sysClassSaveReq) {
        return toAjax(sysClassService.save(SysClass.saveBuild(sysClassSaveReq)));
    }

    /**
     * 修改班级管理
     */
    @PreAuthorize("@ss.hasPermi('school:class:edit')")
    @Log(title = "班级管理", businessType = BusinessType.UPDATE)
    @PutMapping("/{classId}")
    @ApiOperation("修改班级管理")
    public Result<String> edit(@PathVariable Long classId, @RequestBody SysClassEditReq sysClassEditReq) {
        return toAjax(sysClassService.updateById(SysClass.editBuild(classId,sysClassEditReq)));
    }

    /**
     * 删除班级管理
     */
    @PreAuthorize("@ss.hasPermi('school:class:remove')")
    @Log(title = "班级管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{classIds}")
    @ApiOperation("删除班级管理")
    @ApiImplicitParam(name = "classId", value = "classId", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> classIds) {
        return toAjax(sysClassService.removeBatchByIds(classIds));
    }
}
