package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.Result;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SysClazz;
import com.ruoyi.system.domain.req.SysClazzQueryReq;
import com.ruoyi.system.domain.req.SysClazzSaveReq;
import com.ruoyi.system.domain.req.SysClazzEditReq;
import com.ruoyi.system.service.SysClazzService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 班级Controller
 *
 * @author ruoyi
 * @date 2024-04-22
 */
@Api(tags = "班级")
@RestController
@RequestMapping("/system/clazz")
public class SysClazzController extends BaseController {
    @Autowired
    private SysClazzService sysClazzService;

    /**
     * 查询班级列表
     */
    @ApiOperation("获取班级列表")
    @PreAuthorize("@ss.hasPermi('system:clazz:list')")
    @GetMapping("/list")
    public Result<TableDataInfo<SysClazz>> list(SysClazzQueryReq sysClazzQueryReq) {
        startPage();
        List<SysClazz> list = sysClazzService.list(SysClazz.queryBuild(sysClazzQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出班级列表
     */
    @ApiOperation("导出班级列表")
    @PreAuthorize("@ss.hasPermi('system:clazz:export')")
    @Log(title = "班级", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysClazz sysClazz) {
        List<SysClazz> list = sysClazzService.list(sysClazz);
        ExcelUtil<SysClazz> util = new ExcelUtil<SysClazz>(SysClazz.class);
        util.exportExcel(response, list, "班级数据");
    }

    /**
     * 获取班级详细信息
     */
    @ApiOperation("获取班级详细信息")
    @PreAuthorize("@ss.hasPermi('system:clazz:query')")
    @GetMapping(value = "/{clazzId}")
    @ApiImplicitParam(name = "clazzId", value = "clazzId", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<SysClazz> getInfo(@PathVariable("clazzId") Long clazzId) {
        return Result.success(sysClazzService.getById(clazzId));
    }

    /**
     * 新增班级
     */
    @PreAuthorize("@ss.hasPermi('system:clazz:add')")
    @Log(title = "班级", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增班级")
    public Result<String> add(@RequestBody SysClazzSaveReq sysClazzSaveReq) {
        return toAjax(sysClazzService.save(SysClazz.saveBuild(sysClazzSaveReq)));
    }

    /**
     * 修改班级
     */
    @PreAuthorize("@ss.hasPermi('system:clazz:edit')")
    @Log(title = "班级", businessType = BusinessType.UPDATE)
    @PutMapping("/{clazzId}")
    @ApiOperation("修改班级")
    public Result<String> edit(@PathVariable Long clazzId, @RequestBody SysClazzEditReq sysClazzEditReq) {
        return toAjax(sysClazzService.updateById(SysClazz.editBuild(clazzId,sysClazzEditReq)));
    }

    /**
     * 删除班级
     */
    @PreAuthorize("@ss.hasPermi('system:clazz:remove')")
    @Log(title = "班级", businessType = BusinessType.DELETE)
	@DeleteMapping("/{clazzIds}")
    @ApiOperation("删除班级")
    @ApiImplicitParam(name = "clazzId", value = "clazzId", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> clazzIds) {
        return toAjax(sysClazzService.removeBatchByIds(clazzIds));
    }
}
